use super::{EntityDatum, EntityDatumList};

impl EntityDatum for String {
    fn sql_type() -> &'static str {
        "text"
    }

    fn bind_to<'a>(&self, stmt: &mut sqlite::Statement<'a>, index: usize) {
        stmt.bind((index, self.as_str()))
            .expect("couldn't bind string");
    }
}

impl EntityDatum for usize {
    fn sql_type() -> &'static str {
        "int"
    }

    fn bind_to<'a>(&self, stmt: &mut sqlite::Statement<'a>, index: usize) {
        stmt.bind((index, *self as i64))
            .expect("couldn't bind usize as i64");
    }
}

impl EntityDatum for isize {
    fn sql_type() -> &'static str {
        "int"
    }

    fn bind_to<'a>(&self, _stmt: &mut sqlite::Statement<'a>, index: usize) {
        todo!()
    }
}

impl EntityDatum for u64 {
    fn sql_type() -> &'static str {
        "int"
    }

    fn bind_to<'a>(&self, _stmt: &mut sqlite::Statement<'a>, index: usize) {
        todo!()
    }
}

impl EntityDatum for i64 {
    fn sql_type() -> &'static str {
        "int"
    }

    fn bind_to<'a>(&self, _stmt: &mut sqlite::Statement<'a>, index: usize) {
        todo!()
    }
}

impl<T: EntityDatum> EntityDatum for Option<T> {
    fn sql_type() -> &'static str {
        T::sql_type()
    }

    fn bind_to<'a>(&self, _stmt: &mut sqlite::Statement<'a>, index: usize) {
        todo!()
    }
}

impl EntityDatumList for () {
    fn accept(&self, _: &mut impl super::EntityDatumListVisitor) {}
}

impl<T: EntityDatum> EntityDatumList for T {
    fn accept(&self, visitor: &mut impl super::EntityDatumListVisitor) {
        visitor.visit(self);
    }
}

impl<T0: EntityDatum> EntityDatumList for (T0,) {
    fn accept(&self, visitor: &mut impl super::EntityDatumListVisitor) {
        visitor.visit(&self.0);
    }
}

impl<T0: EntityDatum, T1: EntityDatum> EntityDatumList for (T0, T1) {
    fn accept(&self, visitor: &mut impl super::EntityDatumListVisitor) {
        visitor.visit(&self.0);
        visitor.visit(&self.1);
    }
}

impl<T0: EntityDatum, T1: EntityDatum, T2: EntityDatum> EntityDatumList for (T0, T1, T2) {
    fn accept(&self, visitor: &mut impl super::EntityDatumListVisitor) {
        visitor.visit(&self.0);
        visitor.visit(&self.1);
        visitor.visit(&self.2);
    }
}

impl<T0: EntityDatum, T1: EntityDatum, T2: EntityDatum, T3: EntityDatum> EntityDatumList
    for (T0, T1, T2, T3)
{
    fn accept(&self, visitor: &mut impl super::EntityDatumListVisitor) {
        visitor.visit(&self.0);
        visitor.visit(&self.1);
        visitor.visit(&self.2);
        visitor.visit(&self.3);
    }
}
